import { jsx as _jsx } from "react/jsx-runtime";
import React, { useCallback, useEffect, useRef, useState } from 'react';
import { Text } from 'slate';
import { Editable, Slate } from 'slate-react';
import { genToken } from '@sinoform/helper-formula-analysis';
import styled from 'styled-components';
import { useStore } from 'react-redux';
import isEqual from 'lodash/isEqual';
import { DEFAULT_CELL_VALUE, findAllCodes, } from '@sinoform/print-out-core';
import Leaf from './Leaf';
import autoSelectCodeBlock from '../../utils/autoSelectCodeBlock';
import { EDITOR_CHANGE, eventbus } from '../../constants';
import isCodeType from '../../utils/isCodeType';
import useWatchProp from '../../utils/useWatchProp';
import getCellValue from '../../utils/getCellValue';
const EditableLayout = styled(Editable) `
  min-width: 0px;
  transition: box-shadow 0.2s, min-width 0.2s, outline 0.2s;

  background-color: ${({ bgColor, theme }) => bgColor || theme.palette.background.paper};

  ${({ readOnly, theme, width }) => !readOnly &&
    width &&
    width < 400 &&
    `
    position: absolute !important;
    min-width:400px;
    min-height:40px;
    max-width:800px;
    z-index:1;
    box-shadow:${theme.shadows[8]};
    padding:4px;
    border-radius:2px;
    outline: 2px solid ${theme.palette.primary.main} !important;
    top:0;
    left:0;
  `}
  ${({ readOnly, width }) => !readOnly &&
    width &&
    width >= 400 &&
    `
    width:100%;
  `};
`;
/**
 * 文本编辑器
 * @param props
 * @returns
 */
function SlateEditor(props) {
    const { value: valueProp, bgColor, width, fields, rowIdx, colIdx, editor, readOnly, ...other } = props;
    const state = useStore().getState().present;
    const [value, setValue] = useState(valueProp ?? DEFAULT_CELL_VALUE);
    useWatchProp(valueProp ?? DEFAULT_CELL_VALUE, setValue);
    const currentSelection = useRef({});
    useEffect(() => {
        if (editor.selection) {
            currentSelection.current = editor.selection;
        }
    }, [editor.selection]);
    const handleChange = useCallback((newValue) => {
        if (editor.selection &&
            !isEqual(editor.selection, currentSelection.current)) {
            currentSelection.current = editor.selection;
            eventbus.emit(EDITOR_CHANGE, editor);
        }
        setValue(newValue);
    }, [editor]);
    useWatchProp(readOnly, (isReadonly) => {
        if (isReadonly) {
            const prevValue = getCellValue(state, rowIdx, colIdx);
            if (prevValue !== value) {
                setValue(prevValue);
            }
        }
    });
    const handleKeyDown = useCallback((event) => {
        if (event.key === 'Enter' &&
            isCodeType(editor, editor.selection.focus)) {
            event.preventDefault();
        }
    }, [editor]);
    const handleMouseUp = useCallback(() => {
        autoSelectCodeBlock(editor);
    }, [editor]);
    const decorate = useCallback(([node, path]) => {
        const ranges = [];
        if (Text.isText(node)) {
            const { text } = node;
            const codeBlocks = findAllCodes(text);
            codeBlocks.forEach((codeBlock) => {
                const { start } = codeBlock;
                const tokens = genToken(codeBlock.text, fields);
                // eslint-disable-next-line no-restricted-syntax
                for (const token of tokens) {
                    if (typeof token !== 'string') {
                        ranges.push({
                            type: token.type,
                            anchor: { path, offset: token.start + start },
                            focus: { path, offset: token.end + start },
                            error: token.error,
                            title: token.title,
                        });
                    }
                }
            });
        }
        return ranges;
    }, [fields]);
    const renderLeaf = useCallback((leafProps) => _jsx(Leaf, { ...leafProps }), []);
    return (_jsx(Slate, { editor: editor, value: value, onChange: handleChange, children: _jsx(EditableLayout, { ...other, width: width, decorate: decorate, renderLeaf: renderLeaf, onMouseUp: handleMouseUp, onKeyDown: handleKeyDown, bgColor: bgColor, style: { outline: '0px solid transparent' }, spellCheck: true }) }));
}
export default React.memo(SlateEditor);
