import { DEFAULT_CELL_WIDTH, PAGE_SIZE_CONFIG, DEFAULT_CELL_HEIGHT, mmToPx, } from '@sinoform/print-out-core';
/**
 * 判断一个单元格是否被合并，用于单元格渲染时的判断条件
 * 如果被合并返回true，将不再被渲染
 * 需要注意的是，合并单元格第一个位置上的单元格因为需要被渲染，所以返回false
 * @param rowIdx  单元格所在行
 * @param colIdx  单元格所在列
 * @param merges  所有的合并单元格信息
 * @returns
 */
const checkTdIsMerged = (rowIdx, colIdx, merges) => {
    if (!merges) {
        return false;
    }
    const result = merges.find(([[sri, sci], [eri, eci]]) => sri <= rowIdx && eri >= rowIdx && sci <= colIdx && eci >= colIdx);
    if (!result) {
        return false;
    }
    return !(rowIdx === result[0][0] && colIdx === result[0][1]);
};
const getBorderStyle = (type, color) => {
    switch (type) {
        case 'thin':
            return `1px solid ${color}`;
        case 'medium':
            return `2px solid ${color}`;
        case 'thick':
            return `3px solid ${color}`;
        case 'dashed':
            return `1px dashed ${color}`;
        case 'dotted':
            return `1px dotted ${color}`;
        default:
            return `1px solid ${color}`;
    }
};
/**
 * 转换单元格的边框样式
 * @param border
 * @returns
 */
const getCellBorderStyle = (border = {}) => {
    const style = {};
    Object.entries(border).forEach(([key, val]) => {
        style[key] = getBorderStyle(val[0], val[1]);
    });
    return style;
};
/**
 * 根据打印配置获取对应的标尺线位置
 * @param page 页面配置信息
 * @param type 水平线或垂直线
 * @returns
 */
const transformCMToPxByConfig = (page, type = 'h') => {
    const { pageConfig: { pageSize, pageDir, pagePaddingLeft, pagePaddingRight, pagePaddingTop, pagePaddingBottom, }, } = page;
    const left = parseFloat(pagePaddingLeft);
    const right = parseFloat(pagePaddingRight);
    const top = parseFloat(pagePaddingTop);
    const bottom = parseFloat(pagePaddingBottom);
    const space = type === 'h' ? top + bottom : left + right;
    return mmToPx(PAGE_SIZE_CONFIG[pageSize][pageDir === '横向' ? 'h' : 'v'][type === 'h' ? 1 : 0] - space);
};
/**
 * 获取单元格的高度
 * @param state
 * @param rowIdx
 * @param cell
 * @returns
 */
const getCellHeight = (state, rowIdx, cell) => {
    let rowHeight = state.rows[rowIdx]?.height ?? DEFAULT_CELL_HEIGHT;
    if (cell?.merge) {
        for (let i = 1; i < cell.merge[0] + 1; i += 1) {
            rowHeight += state.rows[rowIdx + i]?.height ?? DEFAULT_CELL_HEIGHT;
        }
    }
    return rowHeight;
};
/**
 * 获取单元格宽度
 * @param state
 * @param colIdx
 * @param cell
 * @returns
 */
const getCellWidth = (state, colIdx, cell) => {
    let cellWidth = state.cols[colIdx]?.width ?? DEFAULT_CELL_WIDTH;
    if (cell?.merge) {
        for (let i = 1; i < cell.merge[1] + 1; i += 1) {
            cellWidth += state.cols[colIdx + i]?.width ?? DEFAULT_CELL_WIDTH;
        }
    }
    return cellWidth;
};
export { checkTdIsMerged, getCellBorderStyle, transformCMToPxByConfig, getCellHeight, getCellWidth, };
