import { ActionType, } from './types';
/**
 * 合并或拆分单元格
 *
 * 如果选中的单元格中含有被合并的单元格，执行拆分动作，拆分选中区域内所有的被合并的单元格，否则执行合并操作
 *
 * 前提：确保选中单元格的边界是完整的单元格，不能出现被合并的单元格一部分再选中区域内，一部分在选中区域外的情况
 *
 * @param fromCell 选中单元格的起始位置
 * @param toCell   选中单元格的结束位置
 * @returns
 */
export function mergeCells(fromCell, toCell) {
    // 确保起始和结束节点的位置是从左上到右下的方向
    return {
        type: ActionType.MERGE_CELLS,
        payload: {
            fromCell: [
                Math.min(fromCell[0], toCell[0]),
                Math.min(fromCell[1], toCell[1]),
            ],
            toCell: [
                Math.max(fromCell[0], toCell[0]),
                Math.max(fromCell[1], toCell[1]),
            ],
        },
    };
}
/**
 * 拖动改变行高
 * @param rowIdx
 * @param height
 * @returns
 */
export function changeRowHeight(rowIdx, height) {
    return {
        type: ActionType.CHANGE_ROW_HEIGHT,
        payload: {
            rowIdx,
            height,
        },
    };
}
/**
 * 拖动改变列宽
 * @param colIdx
 * @param width
 * @returns
 */
export function changeColWidth(colIdx, width) {
    return {
        type: ActionType.CHANGE_COL_WIDTH,
        payload: {
            colIdx,
            width,
        },
    };
}
/**
 * 新增行
 * @param rowIdx
 * @returns
 */
export function insertRow(rowIdx) {
    return {
        type: ActionType.INSERT_ROW,
        payload: rowIdx,
    };
}
/**
 * 删除行
 * @param from
 * @param len
 * @returns
 */
export function deleteRow(from, len) {
    return {
        type: ActionType.DELETE_ROW,
        payload: { from, len },
    };
}
/**
 * 新增列
 * @param colIdx
 * @returns
 */
export function insertCol(colIdx) {
    return {
        type: ActionType.INSERT_COL,
        payload: colIdx,
    };
}
/**
 * 删除选中的列
 * @param startCol  选中单元格的起始列
 * @param endCol    选中单元格的结束列
 */
export function deleteCol(startCol, endCol) {
    return {
        type: ActionType.DELETE_COL,
        payload: {
            startCol: Math.min(startCol, endCol),
            endCol: Math.max(startCol, endCol),
        },
    };
}
/**
 * 单元格复制
 *
 * @param source 选中的单元格起始和结束位置
 * @param target 目标单元格位置
 * @returns
 */
export function copyCell(source, target) {
    return {
        type: ActionType.COPY_CELL,
        payload: {
            source,
            target,
        },
    };
}
export function cutCell(source, target) {
    return {
        type: ActionType.CUT_CELL,
        payload: {
            source,
            target,
        },
    };
}
export function setPrintConfig(config) {
    return {
        type: ActionType.SET_PRINT_CONFIG,
        payload: config,
    };
}
export function setCellsBorderStyle(range, type, style = ['thin', '#000']) {
    return {
        type: ActionType.SET_CELLS_BORDER_STYLE,
        payload: {
            range,
            type,
            style,
        },
    };
}
export function setCellsStyle(cells, style) {
    return {
        type: ActionType.SET_CELLS_STYLE,
        payload: {
            cells,
            style,
        },
    };
}
